OPTIONS NOMPRINT NONOTES;
/*  This macro takes a dataset of adaptive cluster sample
    results (quadrats as observations, cluster number and
    number of plants within that quadrat) and generates 
    two of Thompsons unbiased estimators of the population
    average.

   The input dataset should have n1 observations, one observation 
   per initial quadrat.  If more than one initial quadrat falls into
   a single network, there should be one observation for each initial
   quadrat, with the same value for &clust_var (the network id) and &Y_k.
   The dataset should have 3 variables:
   a network id (clus_var, defaulting to cluster),
   the number of quadrats included in that network (not counting edge
   quadrats!), &mi (defaulting to _mi), and
   the number of plants in that entire network (Y_k, defaulting to z).

   In order to run this macro, you can include this file in
   your code (e.g., %INCLUDE "acs.sas") or attach your call
   at the bottom of this file.  

   Example use:

DATA test; * From Figure 1 in Philippi;
  INPUT network mi y;
CARDS;
1 6 15
2 1 0
3 1 0
4 1 0
5 1 1
6 1 0
7 1 0
8 1 0
9 3 7
10 1 0
;
%acs(fname=test,
     clus_var=network,
     mi=mi,
     Y_k=y,
     bigN=225,
     outname=cest     
     );

  This version has been recoded to avoid the limitation to 
  cases where (N choose n) is less than 10^306 power, the 
  largest real*8 number on Intel or unix computers.  The calls
  to COMB(m,n) have been replaced with a short loop, where 
  numerator and denominator terms are multiplied in pairs.  The
  major loss is in clarity of code; sorry.  I have not determined 
  the maximum size for the population grid.

  At some point, someone will release S-plus code for the
  Rao-Blackwell version of these estimators.  This macro
  is intended to simplify computations until that time.  You might
  check http://darwin.fiu.edu to see if I've coded this in Splus
  (or, more likely, R).  Rao-Blackwell versions are unlikely, however.
   
version history:

0.9:  1 April 2004    working version, used COMB function, thus
                      limited to bigN ~150^2

1.0   1 August 2004   Factorials expanded and cancelled, so loop
                      over x[i] replaces COMB.  bigN at least
                      500^2 possible.

    Tom Philippi
    Biological Sciences
    Florida International University
    (305) 348-1876
    philippi@fiu.edu

  Permission is given to anyone to freely copy, use, and 
  modify this code.  If you find and fix errors, or if
  you extend the code with additional features or error-
  trapping, I would appreciate it if you would send me 
  a copy of your modifications.

  If you use this code in a publication, please cite
  the paper in Ecology (if it happens).

*/


%MACRO acs(
     fname=rfile,      /* SAS dataset with sampling results */
     clus_var=cluster, /* name of variable for cluster membership */
     mi=_mi,           /* variable with number of quadrats within network */
     Y_k=z,            /* name of variable for quadrat counts */
	 bigN=,            /* total number of quadrats in sample space */
     outname=cest      /* SAS dataset for estimators */ 
     );

* Count n1, the number of quadrats in the initial sample,
  from the number of observations in the initial dataset;
DATA _NULL_;
  IF (6 EQ 9) THEN SET &fname NOBS=nobs;
  CALL SYMPUT("n1",nobs);
RUN;

* mu_tilde, the number of initial intersections
  Hansen-Hurwitz estimator;
* Thompson & Seber eqn. 4.15;

* Convert network sums and numbers of quadrats into means;
DATA _hh;
  SET &fname;
  _wi = &Y_k / &mi;
RUN;
* Compute HH estimator as mean of network means;
PROC MEANS DATA=_hh NOPRINT;
  VAR _wi;
  OUTPUT OUT=_mutilde MEAN=_mu_HH;
RUN;
* Stick HH mean into macro variable;
DATA _NULL_;
  SET _mutilde;
  CALL SYMPUT("mu_HH",_mu_HH);
RUN;
* Compute var of _mu_HH;
DATA _NULL_;
  RETAIN vsum 0;
  SET _hh END=fini;
  vsum = vsum + (_wi - &mu_HH ) **2;
  IF (fini) THEN DO;
    _weight = (&bigN - &n1) / (&bigN * &n1 * (&n1 - 1));
    vsum = vsum * _weight;
*	PUT vsum=;
	CALL SYMPUT("var_HH",vsum);
  END; * if fini;

* mu_hat, the initial intersection probability 
  Horvitz-Thompson estimator;
* Only use each network once, no matter how many times intersected;
PROC SORT DATA=&fname OUT=_nodup NODUPKEYS;
  BY &clus_var;
RUN;
DATA _NULL_;
  IF (6 EQ 9) THEN SET _nodup NOBS=nobs;
  CALL SYMPUT("n_unique",nobs);
RUN;

DATA _hat; 
  RETAIN _hat_sum 0 _v_sum 0;
  ARRAY _alpha1_ {&n1} _TEMPORARY_;
  ARRAY _alpha {&n1} _TEMPORARY_;
  ARRAY _x {&n1} _TEMPORARY_;
  ARRAY _Ystar{&n1} _TEMPORARY_;

*  ARRAY _alpha_jk {&n1,&n1} _TEMPORARY_;

  SET _nodup END=fini;
   _x[_N_] = &mi;
  _Ystar[_N_] = &Y_k;
  q = 1;
  _noff = &bigN - _x[_N_] - &n1;
  _doff = &bigN - _x[_N_];
  DO _i = 1 TO _x[_N_];
    q = q * (_noff + _i) / (_doff + _i);
  END;
  _alpha1_[_N_] = q;
  _alpha[_N_] = 1 - q;

  _hat_sum = _hat_sum + &Y_k / _alpha[_N_];
* PUT _N_= _noff= _doff= &Y_k= _alpha[_N_]= +2 _hat_sum=;

  IF (fini) THEN DO;
    DO _j = 1 TO &n_unique;
	  IF (_Ystar[_j] GT 0) THEN DO;
	    DO _k = 1 TO &n_unique;
          IF (_Ystar[_k] GT 0) THEN DO;
*	        PUT _j= _k=;
	        IF (_j NE _k) THEN DO;
              q = 1;
              _noff = &bigN - _x[_j] - _x[_k] - &n1;
              _doff = &bigN - _x[_j] - _x[_k];
		      _lim = (_x[_j] + _x[_k]);
              DO _i = 1 TO _lim;
                q = q * (_noff + _i) / (_doff + _i);
              END;
              _alpha_jk = 1 - (_alpha1_[_j] + _alpha1_[_k] - q);
            END; 
            ELSE DO;
			  _alpha_jk = _alpha[_j];
            END; * if _j NE _k;
            _term1 = _alpha_jk / (_alpha[_j]*_alpha[_k]) - 1;
	        _v_sum = _v_sum + _Ystar[_j] * _Ystar[_k] * _term1 / _alpha_jk;	
          END; * if ystar_k;
        END; * do k;
	  END; * if ystar_j;
	END; *do j;
* _mu_hat;
  _mu_HT = _hat_sum / &bigN;
* var_mu_hat;
  _var_HT = _v_sum / (&bigN **2);
  CALL SYMPUT("mu_HT",_mu_HT);
  CALL SYMPUT("var_HT",_var_HT);
  END; * if fini;
RUN;

DATA &outname;
  mu_HH = &mu_HH;
  var_HH = &var_HH;
  sd_HH = SQRT(var_HH);
  uc_HH = mu_HH + (1.96 * sd_HH);
  lc_HH = mu_HH - (1.96 * sd_HH);
  tot_HH = &mu_HH * &bigN;
  sd_tot_HH = &bigN * sd_HH;
  uc_tot_HH = uc_HH * &bigN;
  lc_tot_HH = lc_HH * &bigN;

  mu_HT = &mu_HT;
  var_HT = &var_HT;
  sd_HT = SQRT(var_HT);
  uc_HT = mu_HT + (1.96 * sd_HT);
  lc_HT = mu_HT - (1.96 * sd_HT);
  tot_HT = &mu_HT * &bigN;
  sd_tot_HT = &bigN * sd_HT;
  uc_tot_HT = uc_HT * &bigN;
  lc_tot_HT = lc_HT * &bigN;
  OUTPUT;
  PUT "Hansen-Hurwitz Estimator";
  PUT +5 mu_HH= var_HH= sd_HH= / +5 tot_HH=  sd_tot_HH=;
  PUT "Horvitz-Thompson Estimator";
  PUT +5 mu_HT= var_HT= sd_HT= / +5 tot_HT=  sd_tot_HT=;
RUN;

%MEND acs;


/*

* Test data;
DATA test; * From Figure 1 in Philippi;
  INPUT network mi y;
CARDS;
1 6 15
2 1 0
3 1 0
4 1 0
5 1 1
6 1 0
7 1 0
8 1 0
9 3 7
10 1 0
;
%acs(fname=test,
     clus_var=network,
     mi=mi,
     Y_k=y,
     bigN=225,
     outname=cest     
     );

DATA test2; * From Thompson & Seber 1996 p112-117;
 INPUT network mi y;
  CARDS;
1 6 36
2 11 107
3 1 0
4 1 0
5 1 0
6 1 0
7 1 0
8 1 0
9 1 0
10 1 0
;
%acs(fname=test2,
     clus_var=network,
     mi=mi,
     Y_k=y,
     bigN=400,
     outname=cest2     
     );
*/

